/* 
 Standard Bag of features code
 =============================
 This is modified version of the original cbcreate code for ALADDIN. 
 It expects a matrix of datapoints as input file and returns the centers
 after k-means clustering. 

 This does not perform quantization.

 subh@cs.ucf.edu 
*/
#include "codebook.h"
int main(int argc, char **argv)
{
  /* Input Data Related variables */
  P_PARAMS cp;
  float *data=NULL;
  long int nDataPts =0, nDataDim =0;
  int *data_count=NULL;
  /* Codebook related variables */
  float *centers_data=NULL, *data_dists=NULL;
  int *memberships=NULL;
   
  if (parse_options (argc, argv, &cp) !=0)
    return 1;
  
  /* Read Inputs dataset file, data points in training or validation */
  data = parse_data_file (cp.data_file, &nDataPts, &nDataDim);
  fprintf (stderr, "INF:\n" 
                   "  Total Number of datapoints : %ld\n"
                   "  Datapoint Dimension        : %ld\n", 
      nDataPts, nDataDim);


  /* allocating storage for cluster memberships and distances */
  data_dists = (float *)  malloc (nDataPts * sizeof(float));
  memberships = (int *) malloc (nDataPts * sizeof (int));

  centers_data = (float *)  malloc (cp.nclusters * nDataDim * sizeof(float));
  /* Create codebook using input data */
  create_codebook (data, nDataPts, nDataDim, cp.nclusters, centers_data, 
    memberships, data_dists);
  /* write codebook in codebook file as specified */
  write_data (cp.nclusters, nDataDim, centers_data, cp.codebook_file);
 
  free (data_count);
  free (data);
  free (memberships);
  free (centers_data);
  free (data_dists);
  return 0;
}
/* Perform k-means clustering or any sophisticated clustering, generate the 
 codebook containing the cluster centers. The output is cluster centers and the
 memberships of the individual data-points for building Bag-of-words 
 representation*/
void create_codebook (float *data, long int nDataPts, 
    long int nDataDim, int nclusters, float *centers_data, int *memberships, 
    float *data_dists) {
      
  float speedup; 
  FLANNParameters p_clus, p_idx, p_search;
  FLANN_INDEX centers_idx;
      
  //Initialize FLANN clustering parameters 
  init_FLANN_clustering (&p_clus);
  flann_compute_cluster_centers (data, nDataPts, nDataDim, nclusters, 
    centers_data, &p_clus);
   
  /*Build index for codebook, each datapoint is a cluster center in the 
  indexing structure */
  init_FLANN_indexing (&p_idx);
  centers_idx = flann_build_index (centers_data, nclusters, nDataDim, 
    &speedup, &p_idx);
  /*Search for Nearest neighbors to find the cluster assignments*/
  init_FLANN_searching (&p_search);
  flann_find_nearest_neighbors_index (centers_idx, data, nDataPts,
    memberships, data_dists, NN, &p_search);
  flann_free_index (centers_idx, &p_idx);
}  
void usage (char* pname) {
  fprintf (stderr, "Usage: %s\n"
     "  -n #cluster       [Input number]\n" 
     "  -d <datafile>     [Input path of File with descriptors]\n"
     "  -b <codebook file>[Output path of codebook file]\n", pname);
}

int parse_options (int argc, char *argv[], P_PARAMS *p) {
  int i;
  
  if( argc == 1) {
    usage (argv[0]);
    return 1;
  }
  for (i = 1; i < argc; i++ ){
    if (!strcmp (argv[i], "-n")) {
      p->nclusters = strtod (argv[++i], NULL);
    } 
    else if (!strcmp (argv[i], "-d")) /* Data file */
      strcpy (p->data_file , argv[++i]);
    else if (!strcmp (argv[i], "-b"))  /*code book file RW*/
      strcpy (p->codebook_file,  argv[++i]);
  }
  fprintf (stderr, "INF:Using parameters\n");
        
  fprintf (stderr, "  Number of Clusters       : %d\n"
                   "  Output codebook file     : %s\n",
        p->nclusters, p->codebook_file);
  
  return 0;
}

void init_FLANN_clustering (FLANNParameters *p) {
  // FLANN clustering parameters
  p->algorithm = FLANN_INDEX_KMEANS;
  p->checks = 2048;
  p->cb_index = 0.6;
  //p->trees = 64; //(rec val)
  p->branching = 2;
  p->iterations = 1000;
  p->centers_init = FLANN_CENTERS_GONZALES;
  p->target_precision = -1;
  p->build_weight = 0.01;
  p->memory_weight = 1;
}
void init_FLANN_indexing (FLANNParameters* p) {
  p->algorithm = FLANN_INDEX_KDTREE;
  p->checks =2048;
  p->trees = 16;
  p->target_precision = -1;
  p->build_weight = 0.01;
  p->memory_weight = 1;
}
void init_FLANN_searching (FLANNParameters* p){
  p->log_level = FLANN_LOG_ERROR;
  p->random_seed = FLANN_CENTERS_RANDOM; 
}
