/* 
 Standard Bag of features code
 =============================
 Please refer to README for more details.
 subh@cs.ucf.edu 
*/
#include "codebook.h"

//Linearly indexed data (faster access)
float* parse_data_file (char *filepath, long int *nDataPts, long int *nDataDim){
  FILE *fp;
  char line[MAX_LINE_WIDTH];
  char *token;
  // Data is loaded into 1D floating point array, we don't use double here as 
  // libflann uses float, and cvKmeans uses CV_32FC1
  float *data;
  long int jcnt, icnt;
  long int total_elements;
   
  fp = fopen (filepath, "r");
  if (!fp) {
    fprintf (stderr, "ERR:Cannot open data file %s.\n", filepath);
    return NULL;
  }
  // Find out the data dimensions
  fgets (line, MAX_LINE_WIDTH, fp);

  if (strlen (line) == MAX_LINE_WIDTH-1) {
    fprintf (stderr, "ERR:Incorrect line read(%d bytes). Please increase \
MAX_LINE_WIDTH by a factor of 2 and recompile.\n", MAX_LINE_WIDTH-1);
    return NULL;
  }
  strtok (line, DELIM);
  ++*nDataDim;
  while (strtok(NULL, DELIM))
    ++*nDataDim;
  ++*nDataPts;
  // Find out number of data points
  while (fgets(line, MAX_LINE_WIDTH, fp))
    ++*nDataPts;
  
  rewind(fp);
  total_elements = (*nDataPts) * (*nDataDim);
  // Allocate memory and read data
  data  = (float *) malloc ( total_elements* sizeof(float));
          
  for (icnt =0; icnt < *nDataPts; icnt++ ) {
    //sanity check 
    if (!data){
      fprintf (stderr, "ERR: Unable to allocate memory.\n");
      return NULL;
    }  
    // read a line parse a token store it in the array
    fgets (line, MAX_LINE_WIDTH, fp);
    token = strtok (line, DELIM);
    data[icnt*(*nDataDim)+0] = strtof (token, NULL);
      
    jcnt = 1;
    while ((token = strtok (NULL, DELIM))!=NULL){
      data[icnt*(*nDataDim) + jcnt] = strtof (token, NULL);
      jcnt++;
    }
  }
  //clean up
  fclose (fp);
  return data;
}
int *read_data_count (char *filepath, int *num_training) {
  FILE *fp;
  int BUFF_SIZE = 32, i =0;
  int *data_count;
  char line[BUFF_SIZE];
  
  fp = fopen (filepath, "r");
  if (!fp) {
    fprintf (stderr, "ERR:Cannot open data file %s.\n", filepath);
    return NULL;
  }
  // Count number of training samples provided.
  *num_training = 0;
  while (fgets(line, BUFF_SIZE, fp))
    ++*num_training;
  rewind(fp);
  
  data_count = (int *) malloc (*num_training*sizeof (int));
  // Populate the data_count array  
  while (fgets(line, BUFF_SIZE, fp))
    data_count[i++] = (int) strtod (line, NULL);
  fclose (fp);
  
  return data_count;
}
void write_data (long int rows, long int cols, float *data, char *fname){
  FILE *fp;
  int icnt, jcnt; 
  
  if (fname){
    fp = fopen (fname, "w+");
    if (!fp) {
      fprintf (stderr, "ERR:Cannot write into file:%s.Using stdout.\n", fname);
      fp = stdout;
    }
  } else
    fp = stdout;
  
  fprintf (stderr, "INF:Data for %ld x %ld data matrix\n", rows, cols);
  for (icnt =0;icnt < rows; icnt++) {
    for (jcnt =0;jcnt < cols-1; jcnt++)
      fprintf (fp, "%8.7f ", data[icnt*cols + jcnt]);
    fprintf (fp, "%8.7f\n", data[icnt*cols + cols-1]);
  }
  if (fname) 
    fclose (fp);
}
void print_matrix (CvMat *dataMat, char *fname){
  long int icnt, jcnt;
  long int data_pts = dataMat->rows;
  long int data_dim = dataMat->cols;
  FILE *fp;
  
  if (fname) 
    fp = fopen (fname, "w+"); 
  else 
    fp = stdout;   
  
  for (icnt=0;icnt < data_pts; icnt++) {
    for (jcnt=0; jcnt <data_dim-1; jcnt++)
      fprintf (fp, "%8.7f ", cvmGet (dataMat, icnt, jcnt));
    fprintf (fp, "%8.7f\n", cvmGet (dataMat, icnt, data_dim -1));
  }

  if (fname) 
    fclose (fp);
}
